rm(list=ls())

#function for generating dataset
generate.data <- function(n=100, p=2, noise=0.05){
  
  X <- data.frame(lapply(1:p,function(x){runif(n)}))
  colnames(X) <- paste("x",1:p,sep="")
  
  y.clean <- ifelse(X$x2 > 4*X$x1,1,2)
  
  y <- factor(ifelse(runif(n)>(1.0-noise),3-y.clean,y.clean))
  levels(y) <- c("neg","pos")
  
  all.data <- cbind(y,X)
  
  return(all.data)
  
}

#number of descriptors
p <- 2

#training set size
n.train <- 300 

#test set size
n.test <- 20000

#noise on the class attribute - theoretical error rate
noise <- 0.05

#initialize the random number generator
set.seed(10)

#test set
test.data <- generate.data(n.test,p,noise)

#training set
train.data <- generate.data(n.train,p,noise)

#plotting training set
plot(train.data$x1,train.data$x2,pch=21,bg=c("red","blue")[unclass(train.data$y)])

#plotting test set
plot(test.data$x1,test.data$x2,pch=21,bg=c("red","blue")[unclass(test.data$y)])

#function for computing error rate with plotting
error.rate.plot <- function(data.test,pred){
  plot(data.test$x1,data.test$x2,pch=21,bg=c("red","blue")[unclass(pred)])
  abline(0,4,col="green",lwd=5)
  mc <- table(data.test$y,pred)
  print(mc)
  err.rate <- 1-sum(diag(mc))/sum(mc)
  return(err.rate)
}

#theoretical prediction
pred.thq <- factor(ifelse(test.data$x2-4*test.data$x1>0,1,2))
print(error.rate.plot(test.data,pred.thq))

#conditional density plot
library(lattice)
densityplot(test.data$x1,groups=test.data$y)
densityplot(test.data$x2,groups=test.data$y)

#naive bayes classifier
library(e1071)
model.nb <- naiveBayes(y ~ ., data = train.data)
print(model.nb)

#function for naive bayes prediction
prediction.nb <- function(model,test.data){
  return(predict(model,newdata=test.data))
}

print(error.rate.plot(test.data,prediction.nb(model.nb,test.data)))

#linear discriminant analysis
library(MASS)
model.lda <- lda(y ~ ., data = train.data)
print(model.lda)

#function for linear discriminant analysis prediction
prediction.lda <- function(model,test.data){
  return(predict(model,newdata=test.data)$class)
}

print(error.rate.plot(test.data,prediction.lda(model.lda,test.data)))

#

#logistic regression
model.glm <- glm(y ~ ., data = train.data, family = binomial)
print(summary(model.glm))

#function for logistic regression prediction
prediction.glm <- function(model,test.data){
  return(factor(ifelse(predict(model,newdata=test.data)>0.5,2,1)))
}

print(error.rate.plot(test.data,prediction.glm(model.glm,test.data)))

#single layer perceptron (neural network)
library(nnet)
model.nn <- nnet(y ~ ., data = train.data,skip=TRUE,size=0)
print(summary(model.nn))

prediction.nn <- function(model,test.data){
  return(factor(predict(model,newdata=test.data,type="class")))
}

print(error.rate.plot(test.data,prediction.nn(model.nn,test.data)))

#linear support vector machine
library(e1071)
model.svm <- svm(y ~ ., data = train.data,kernel="linear")
print(model.svm)

#function for svm prediction
prediction.svm <- function(model,test.data){
  return(predict(model,newdata=test.data))
}

print(error.rate.plot(test.data,prediction.svm(model.svm,test.data)))

plot(model.svm,data=train.data,svSymbol="x",dataSymbol="o")


#decision tree learning
library(rpart)
model.tree <- rpart(y ~ ., data = train.data)
print(model.tree)
pred.tree <- predict(model.tree,newdata=test.data,type="class")
print(error.rate.plot(test.data,pred.tree))

#random forest
library(randomForest)
model.rf <- randomForest(y ~ ., data = train.data)
print(model.rf)
pred.rf <- factor(predict(model.rf,newdata=test.data,type="response"))
print(error.rate.plot(test.data,pred.rf))

#nearest neighbor
library(class)
#k=1
print(error.rate.plot(test.data,knn(train.data[,2:3],test.data[,2:3],train.data$y,k=1)))
#k=5
print(error.rate.plot(test.data,knn(train.data[,2:3],test.data[,2:3],train.data$y,k=5)))

#exporting the training set
write.table(train.data,file="training_set.txt",sep="\t",dec=".",quote=F,row.names=F)


#*******************************************
#EXPERIMENTS - VARYING THE TRAINING SET SIZE
#*******************************************

#function for computing error rate without plotting
error.rate <- function(data.test,pred){
  mc <- table(data.test$y,pred)
  err.rate <- 1-sum(diag(mc))/sum(mc)
  return(err.rate)
}

#experiments for the size "n" of the learning set
#evaluating the performance of the various learning algorithms
experiments <- function(n,test.set){
  
  #learning set
  learning <- generate.data(n,p,noise)
  print(nrow(learning))
  
  #vector containing the results
  result <- numeric(5)
  
  #naive bayes classifier
  model.nb <- naiveBayes(y ~ ., data = learning)
  result[1] <- error.rate(test.set,prediction.nb(model.nb,test.set))  
  
  #linear discriminant analysis
  model.lda <- lda(y ~ ., data = learning)
  result[2] <- error.rate(test.set,prediction.lda(model.lda,test.set))
  
  #logistic regression
  model.glm <- glm("y ~ .", data = learning, family = binomial)
  result[3] <- error.rate(test.set,prediction.glm(model.glm,test.set))
  
  #single layer perceptron
  model.nn <- nnet(y ~ ., data = learning,skip=TRUE,size=0)
  result[4] <- error.rate(test.set,prediction.nn(model.nn,test.set))
  
  #linear support vector machine
  model.svm <- svm(y ~ ., data = learning,kernel="linear")
  result[5] <- error.rate(test.set,prediction.svm(model.svm,test.set))
  
  print(result)

  return(result)
  
}


#various training set size
size.training <- c(250,500,1000,2000,3000,5000)

#initialize the random number generator
set.seed(25032003)
#test data
other.data.test <- generate.data(100000,p,noise)

#starting the experiments
library(MASS)
library(e1071)
library(nnet)

one.expe.session <- function(size.learning){
  results <- mapply(experiments,size.learning,MoreArgs=list(test.set=other.data.test))
  return(results)
}

#one experiment for the checking of the results
#set.seed(05092008)
#one.result <- one.expe.session(size.training)

#repeat K times the experiments
K <- 100
set.seed(05092008)
all.results <- replicate(K,one.expe.session(size.learning=size.training),simplify="matrix")

#retrieving the results
mean.results <- matrix(0,nrow=5,ncol=length(size.training))
colnames(mean.results) <- size.training
rownames(mean.results) <- c("naive.bayes","lda","log.reg","perceptron","svm.linear")

for (i in 1:5){
  for (j in 1:length(size.training)){
    mean.results[i,j] <- mean(all.results[i+(j-1)*5,])
  }
}

print(mean.results)

#median.results
med.results <- mean.results
for (i in 1:5){
  for (j in 1:length(size.training)){
    med.results[i,j] <- median(all.results[i+(j-1)*5,])
  }
}

print(med.results)


#**********************************************
# ANALYZING THE INFLUENCE OF THE DIMENSIONALITY
#**********************************************

#experiments for the dimension "p" of the learning set
#evaluating the performance of the various learning algorithms
#learning set size = 500
experiments.dimension <- function(p,test.set){
  
  #learning set
  learning <- generate.data(500,p,noise)
  print(nrow(learning))
  print(ncol(learning)-1)
  
  #vector containing the results
  result <- numeric(5)
  
  #naive bayes classifier
  model.nb <- naiveBayes(y ~ ., data = learning)
  result[1] <- error.rate(test.set,prediction.nb(model.nb,test.set))  
  print("ok NB")
  
  #linear discriminant analysis
  model.lda <- lda(y ~ ., data = learning)
  result[2] <- error.rate(test.set,prediction.lda(model.lda,test.set))
  print("ok LDA")
  
  #logistic regression
  model.glm <- glm("y ~ .", data = learning, family = binomial)
  result[3] <- error.rate(test.set,prediction.glm(model.glm,test.set))
  print("ok GLM")
  
  #single layer perceptron
  model.nn <- nnet(y ~ ., data = learning,skip=TRUE,size=0)
  result[4] <- error.rate(test.set,prediction.nn(model.nn,test.set))
  print("ok NNET")
  
  #linear support vector machine
  model.svm <- svm(y ~ ., data = learning,kernel="linear")
  result[5] <- error.rate(test.set,prediction.svm(model.svm,test.set))
  print("ok SVM")
  
  print(result)
  
  return(result)
  
}

#specifying a new test set
#initialize the random number generator
set.seed(25032003)
second.data.test <- generate.data(100000,100,noise)

#print
print(colnames(second.data.test))
print(table(second.data.test$y))

#various dimension size
size.p <- c(2,5,10,25,50,70)

#one experiment for various diemnsionality
one.expe.dimension <- function(size.dimension){
  results <- mapply(experiments.dimension,size.dimension,MoreArgs=list(test.set=second.data.test))
  return(results)
}

#one experiment for the checking of the results
#set.seed(21102011)
#one.result <- one.expe.dimension(size.p)

#repeat K times the experiments
K <- 100
set.seed(21102011)
all.results <- replicate(K,one.expe.dimension(size.dimension=size.p),simplify="matrix")

#retrieving the results
mean.results <- matrix(0,nrow=5,ncol=length(size.p))
colnames(mean.results) <- size.p
rownames(mean.results) <- c("naive.bayes","lda","log.reg","perceptron","svm.linear")

for (i in 1:5){
  for (j in 1:length(size.p)){
    mean.results[i,j] <- mean(all.results[i+(j-1)*5,])
  }
}

print(mean.results)

#median.results
med.results <- mean.results
for (i in 1:5){
  for (j in 1:length(size.p)){
    med.results[i,j] <- median(all.results[i+(j-1)*5,])
  }
}

print(med.results)

